# Generated by Django 3.2 on 2022-11-23 11:21

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apis_entities', '0004_auto_20200722_1231'),
        ('dubletten_tool', '0006_delete_ampel'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='vorfin',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_entities.person'),
        ),
    ]
