
from .error_logger_mixin_test import ErrorLoggerMixinTest
from .merge_group_test import MergeGroupTest
from .person_helper_test import PersonHelperTest
from .pphelper_test import PPHelperTest
from .person_type_test import PersonTypeTest
from .ppkind_test import PPKindTest
from .relation_writer_test import RelationWriterTest
from .single_merge_test import SingelMergeTest
from .temp_rel_test import TempRelTest
