# Generated by Django 3.2 on 2022-10-26 21:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0004_auto_20221026_1855'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ampel',
            name='status',
            field=models.CharField(choices=[('red', 'red'), ('yellow', 'yellow'), ('green', 'green')], default='red', max_length=300),
        ),
    ]
