

class Merger: 
    """
    HelperClass Storing common logic and functions used in remerge workflows. 
    Allows for creating seperate Workflows for different kind of remerges.
    """

    def __init__(self, *args, **kwargs):
        pass

    