from django.test import TestCase



class ErrorLoggerMixinTest(TestCase):

    def setup(self, *args, **kwargs):
        pass
    
    def test_class_setup(self, *args, **kwargs):
        pass

    def tets_class_reset(self, *args, **kwargs):
        pass

    