# Generated by Django 3.2 on 2022-11-22 08:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0005_alter_ampel_status'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Ampel',
        ),
    ]
