# Generated by Django 3.2 on 2022-10-26 18:55

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('apis_entities', '0004_auto_20200722_1231'),
        ('dubletten_tool', '0003_ampel'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ampel',
            name='id',
        ),
        migrations.AddField(
            model_name='ampel',
            name='status',
            field=models.CharField(choices=[('red', 'red'), ('yellow', 'yelow'), ('green', 'green')], default='red', max_length=300),
        ),
        migrations.AlterField(
            model_name='ampel',
            name='person',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='apis_entities.person'),
        ),
    ]
